# -*- coding: utf-8 -*-

#importation du package
import h2o

#démarrage H2O
h2o.init(nthreads=-1)

#####################################
#### IMPORTATION DES DONNEES ########
#####################################

#changer le répertoire courant
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/h2o")

#chargement des données
pima = h2o.import_file("pima.csv")

#affichage des premières valeurs
print(pima.head().as_data_frame())

#affichage du type
print(type(pima))

#dimension
print(pima.shape)

#affichage de la liste des colonnes
print(pima.col_names)

#diabete est bien un type facteur
pima['diabete'].isfactor()

#nombre de niveau de "diabete"
pima['diabete'].levels()

###################################
#### SUBDIVISION APP -TEST ########
###################################

#subdivision
pimaTrain,pimaTest = pima.split_frame(ratios=[0.67],seed=1)

#vérification train
pimaTrain.shape

#vérification test
pimaTest.shape

###################################
#### REGRESSION LOGISTIQUE ########
###################################

#régression généralisée
from h2o.estimators import H2OGeneralizedLinearEstimator

#régression logistique
reg = H2OGeneralizedLinearEstimator(seed=100,family="binomial",standardize=True,lambda_=0,compute_p_values=True)

#modélisation
reg.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#affichage par défaut
reg.show()

#affichage des coefficients
reg.coef()

#affichage des coefficients normalisés
reg.coef_norm()

#affichage graphique des coefficients normalisées
reg.std_coef_plot()

#affichage complet avec les tests de significativité
reg.pprint_coef()

#accéder à un champ en particlier, ex. ecarts-type
reg._model_json['output']['coefficients_table']['std_error']

#test de significativité
#stat. test du rapport de vraisemblance
LR = reg.null_deviance() - reg.residual_deviance()
print(LR)

#degrés de liberté
ddl = reg.null_degrees_of_freedom() - reg.residual_degrees_of_freedom()
print(ddl)

#librairie scipy
import scipy.stats as stats

#p-value de significativité globale
pvalue = 1.0 - stats.chi2.cdf(x=LR,df=ddl)
print(pvalue)

#evaluation des performances sur l'échantillon test
reg.model_performance(pimaTest)

#prediction - conversion en format data frame
#attention au seuil utilisé pour l'affectation
predReg = reg.predict(pimaTest).as_data_frame()
predReg.head(20)

#matrice de confusion en test -- avec le seuil de H20
import pandas
pandas.crosstab(pimaTest.as_data_frame()["diabete"],predReg.predict)

#scikit-learn
from sklearn import metrics

#taux d'erreur
print(1-metrics.accuracy_score(pimaTest.as_data_frame()["diabete"],predReg.predict))

#F1-Score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predReg.predict,pos_label="positive"))

#prédiction avec le seuil = 0.5
import numpy
predReg2 = numpy.repeat("negative",pimaTest.shape[0])
predReg2[predReg.positive > 0.5] = "positive"

#matrice de confusion en test -- seuil = 0.5
pandas.crosstab(pimaTest.as_data_frame()["diabete"],predReg2)

#taux d'erreur
print(1-metrics.accuracy_score(pimaTest.as_data_frame()["diabete"],predReg2))

#F1-Score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predReg2,pos_label="positive"))

###########################
#### RANDOM FOREST ########
###########################

#random forest
from h2o.estimators import H2ORandomForestEstimator

#instanciation
rf = H2ORandomForestEstimator(seed=100,ntrees=200,max_depth=50,min_rows=1)

#apprentissage
rf.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#evolution de l'apprentissage
rf.plot()

#résumé
rf.summary()

#affichage
rf.show()

#ou tabulaire
pandas.DataFrame(rf.varimp())

#importance - graphique
rf.varimp_plot()

#evaluation
rf.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predRf = rf.predict(pimaTest).as_data_frame()
print(predRf.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predRf.predict,pos_label="positive"))

###############################
#### GRADIENT BOOSTING ########
###############################

#gradient boosting
from h2o.estimators import H2OGradientBoostingEstimator

#instanciation
gb = H2OGradientBoostingEstimator(seed=100,ntrees=1500,max_depth=5)

#apprentissage
gb.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#évolution
gb.plot()

#affichage
gb.show()

#plotting var importance
gb.varimp_plot()

#evaluation en test
gb.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predGb = gb.predict(pimaTest).as_data_frame()
print(predGb.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predGb.predict,pos_label="positive"))

#########################
#### NAIVE BAYES ########
#########################

#naive bayes
from h2o.estimators import H2ONaiveBayesEstimator

#instanciation
nb = H2ONaiveBayesEstimator(seed=100)

#apprentissage
nb.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#affichage
nb.show()

#evaluation en test
nb.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predNb = nb.predict(pimaTest).as_data_frame()
print(predNb.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predNb.predict,pos_label="positive"))

###############################
#### PERCEPTRON SIMPLE ########
###############################

#deep learning
from h2o.estimators import H2ODeepLearningEstimator

#instanciation
ps = H2ODeepLearningEstimator(seed=100,epochs=1250,standardize=True,hidden=[],distribution="bernoulli", export_weights_and_biases = True)

#apprentissage
ps.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#évolution de l'apprentissage
ps.plot()

#structure du réseau
ps.summary()

#poids synaptiques - coefficients
ps.weights().as_data_frame()

#poids synatpiques- biais
ps.biases().as_data_frame()

#récupération des poids
score = ps.weights().as_data_frame().values

#différences termes à termes - coefficients de la fonction score
fcnScore = score[0,:] - score[1,:]
print(fcnScore)

#index selon la valeur absolue du coefs
index = numpy.argsort(numpy.abs(fcnScore))

#inversion
index = index[::-1]

#affichage
temp = {'variable':ps.weights().as_data_frame().columns[index],'influence':fcnScore[index]}
print(pandas.DataFrame(temp))

#affichage
ps.show()

#importance des variables - graphique
ps.varimp_plot()

#evaluation en test
ps.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predPs = ps.predict(pimaTest).as_data_frame()
print(predPs.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predPs.predict,pos_label="positive"))

####################################
#### PERCEPTRON MULTICOUCHE ########
####################################

#instanciation
pmc = H2ODeepLearningEstimator(seed=100,epochs=1250,standardize=True,hidden=[2],activation="Tanh",distribution="bernoulli", export_weights_and_biases = True)

#apprentissage
pmc.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#structure du réseau
pmc.summary()

#poids - entrée -> cachée
pmc.weights(0).as_data_frame()

#poids - cachée -> sortie
pmc.weights(1).as_data_frame()

#affichage
pmc.show()

#PMC -- coordonnées dans la couche cachée
coordPmc = pmc.deepfeatures(pimaTest,0).as_data_frame()
print(coordPmc.head(10))

#récupération des colonnes
C1 = coordPmc.iloc[:,0]
C2 = coordPmc.iloc[:,1]

#code couleur
couleur = (pimaTest.as_data_frame().loc[:,"diabete"] == "positive").astype('int')

#librairie graphique
import matplotlib.pyplot as plt

#affichage des points dans le plan
plt.scatter(C1,C2,c=pandas.Series(['green','blue'])[couleur],s=8)

#evaluation en test
pmc.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predPmc = pmc.predict(pimaTest).as_data_frame()
print(predPmc.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predPmc.predict,pos_label="positive"))


##########################################################
#### GRADIENT BOOSTING -- AVEC ECH. DE VALIDATION ########
##########################################################

#subdivision
pimaLearning, pimaValidation = pimaTrain.split_frame(ratios=[0.70],seed=1)

#dimensions - learning set
pimaLearning.shape

#dimensions - validation set
pimaValidation.shape

#instanciation
gbBis = H2OGradientBoostingEstimator(seed=100,ntrees=1500,max_depth=5,stopping_metric="log_loss",stopping_rounds=3,stopping_tolerance=1e-3)

#apprentissage
gbBis.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaLearning,validation_frame=pimaValidation)

#évolution
gbBis.plot()

#résumé
gbBis.summary()

#affichage
gbBis.show()

#plotting var importance
gbBis.varimp_plot()

#evaluation en test
gbBis.model_performance(pimaTest)

#prediction - de nouveau voir le seuil d'affectation
predGbBis = gbBis.predict(pimaTest).as_data_frame()
print(predGbBis.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predGbBis.predict,pos_label="positive"))

#####################################################
#### GRADIENT BOOSTING -- VALIDATION CROISEE ########
#####################################################

#instanciation
gbCV = H2OGradientBoostingEstimator(seed=100,nfolds=3)

#apprentissage
gbCV.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#résumé des résultats
gbCV.cross_validation_metrics_summary()

#prediction - de nouveau voir le seuil d'affectation
predGbCV = gbCV.predict(pimaTest).as_data_frame()
print(predGbCV.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predGbCV.predict,pos_label="positive"))

##############################################
#### GRADIENT BOOSTING -- GRID SEARCH ########
##############################################

#définir les paramètres à tester
gbParametres = {'ntrees':[20,50,100,1000] , 'max_depth':[1,2,5,20]}

#classe pour la grille de recherche
from h2o.grid.grid_search import H2OGridSearch

#instancier
gbGrid = H2OGridSearch(model = H2OGradientBoostingEstimator, hyper_params = gbParametres)

#lancer les calculs -- 5-validation croisée
gbGrid.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain,nfolds=5,seed=100)

#affichage des résultats
gbModels = gbGrid.get_grid(sort_by='f1',decreasing=True)
print(gbModels.summary())

#affichage des performances
gbModels.show()

#paramètres du premier (meilleur) modèle
gbModels.get_hyperparams(0)

#récupération du meilleur modele
gbBest = gbModels[0]

#vérification - nombre d'arbre
gbBest.get_params()['ntrees']['actual_value']

#vérification - profondeur max
gbBest.get_params()['max_depth']['actual_value']

#prédiction sur le test
predGbBest = gbBest.predict(pimaTest).as_data_frame()
print(predGbBest.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predGbBest.predict,pos_label="positive"))

#performances du 4e
predGbFour = gbModels[4].predict(pimaTest).as_data_frame()
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predGbFour.predict,pos_label="positive"))

#####################
#### AUTO-ML ########
#####################

#chargement de la classe
from h2o.automl import H2OAutoML

#instanciation
aml = H2OAutoML(seed=100,nfolds=5,max_runtime_secs=180)

#lancement des calculs
aml.train(x=pimaTrain.columns[:-1],y="diabete",training_frame=pimaTrain)

#récupérer le tableau des modèles
lb = aml.leaderboard

#nombre de modèles
print(lb.nrow)

#afficher les modèles -- tri par défaut AUC pour le classement binaire
result = lb.head(rows=lb.nrow).as_data_frame()
result.loc[:,["model_id","auc"]]

#meilleur modèle
aml.leader

#prédiction sur le test
predAml = aml.predict(pimaTest).as_data_frame()
print(predAml.head(10))

#F1-score
print(metrics.f1_score(pimaTest.as_data_frame()["diabete"],predAml.predict,pos_label="positive"))
